@@ -6,26 +6,28 @@ use Carp 'confess';
 use Exporter 'import';
 use JSON::Validator::Error;
 use JSON::Validator::Ref;
+use JSON::Validator::Joi;
 use Mojo::File 'path';
 use Mojo::JSON::Pointer;
 use Mojo::JSON;
 use Mojo::Loader;
 use Mojo::URL;
-use Mojo::Util 'url_unescape';
+use Mojo::Util qw(url_unescape sha1_sum);
 use Scalar::Util qw(blessed refaddr);
 use Time::Local ();
 
-use constant COLORS => eval { require Term::ANSIColor };
-use constant DEBUG => $ENV{JSON_VALIDATOR_DEBUG};
-use constant REPORT => $ENV{JSON_VALIDATOR_REPORT} // $ENV{JSON_VALIDATOR_DEBUG};
-use constant RECURSION_LIMIT => $ENV{JSON_VALIDATOR_RECURSION_LIMIT} || 100;
+use constant CASE_TOLERANT     => File::Spec->case_tolerant;
+use constant COLORS            => eval { require Term::ANSIColor };
+use constant DEBUG             => $ENV{JSON_VALIDATOR_DEBUG};
+use constant REPORT            => $ENV{JSON_VALIDATOR_REPORT} // $ENV{JSON_VALIDATOR_DEBUG};
+use constant RECURSION_LIMIT   => $ENV{JSON_VALIDATOR_RECURSION_LIMIT} || 100;
 use constant SPECIFICATION_URL => 'http://json-schema.org/draft-04/schema#';
 use constant VALIDATE_HOSTNAME => eval 'require Data::Validate::Domain;1';
 use constant VALIDATE_IP       => eval 'require Data::Validate::IP;1';
 
 our $ERR;    # ugly hack to improve validation errors
-our $VERSION = '1.08';
-our @EXPORT_OK = 'validate_json';
+our $VERSION = '2.08';
+our @EXPORT_OK = qw(joi validate_json);
 
 my $BUNDLED_CACHE_DIR = path(path(__FILE__)->dirname, qw(Validator cache));
 my $HTTP_SCHEME_RE = qr{^https?:};
@@ -36,21 +38,9 @@ sub D {
 sub E { JSON::Validator::Error->new(@_) }
 sub S { Mojo::Util::md5_sum(Data::Dumper->new([@_])->Sortkeys(1)->Useqq(1)->Dump) }
 
-has cache_paths => sub {
-  my $self = shift;
-  my @paths = split /:/, ($ENV{JSON_VALIDATOR_CACHE_PATH} || '');
-
-  if ($ENV{JSON_VALIDATOR_CACHE_DIR}) {
-    warn "JSON_VALIDATOR_CACHE_DIR is deprecated in favor of JSON_VALIDATOR_CACHE_PATH\n"
-      unless $ENV{HARNESS_ACTIVE};
-    push @paths, split /:/, ($ENV{JSON_VALIDATOR_CACHE_DIR} || '');
-  }
-
-  push @paths, $BUNDLED_CACHE_DIR;
-  return \@paths;
-};
-
-has formats => sub { shift->_build_formats };
+has cache_paths => sub { [split(/:/, $ENV{JSON_VALIDATOR_CACHE_PATH} || ''), $BUNDLED_CACHE_DIR] };
+has formats     => sub { shift->_build_formats };
+has version     => 4;
 
 has ua => sub {
   require Mojo::UserAgent;
@@ -62,11 +52,10 @@ has ua => sub {
 
 sub bundle {
   my ($self, $args) = @_;
-  my $def_name_cb = $args->{definitions_name} || \&_definitions_name;
   my @topics = ([undef, my $bundle = {}]);
   my ($cloner, $tied);
 
-  $topics[0][0] = $args->{schema} ? $self->_reset->_resolve($args->{schema}) : $self->schema->data;
+  $topics[0][0] = $args->{schema} ? $self->_resolve($args->{schema}) : $self->schema->data;
 
   if ($args->{replace}) {
     $cloner = sub {
@@ -79,16 +68,26 @@ sub bundle {
     };
   }
   else {
-    $bundle->{definitions} ||= {%{$topics[0][0]{definitions} || {}}};
+    my $ref_key = $args->{ref_key} || 'x-bundled';
+    $bundle->{$ref_key} = $topics[0][0]{$ref_key} || {};
     $cloner = sub {
       my $from = shift;
       my $ref  = ref $from;
 
       if ($ref eq 'HASH' and my $tied = tied %$from) {
-        return $from if $tied->fqn =~ m!^\#!;
-        my $name = $self->$def_name_cb($tied->fqn);
-        push @topics, [$tied->schema, $bundle->{definitions}{$name} = {}];
-        tie my %ref, 'JSON::Validator::Ref', $tied->schema, "#/definitions/$name";
+        my $ref_name = $tied->fqn;
+        return $from if $ref_name =~ m!^$self->{root_schema_url}\#!;
+
+        if (-e $ref_name) {
+          $ref_name = sprintf '%s-%s', substr(sha1_sum($ref_name), 0, 10),
+            path($ref_name)->basename;
+        }
+        else {
+          $ref_name =~ s![^\w-]!_!g;
+        }
+
+        push @topics, [$tied->schema, $bundle->{$ref_key}{$ref_name} = {}];
+        tie my %ref, 'JSON::Validator::Ref', $tied->schema, "#/$ref_key/$ref_name";
         return \%ref;
       }
 
@@ -125,14 +124,28 @@ sub coerce {
 
 sub get {
   my ($self, $pointer) = @_;
-  my $data = $self->schema->data;
+  $pointer = [ref $pointer ? @$pointer : length $pointer ? split('/', $pointer, -1) : $pointer];
+  shift @$pointer if @$pointer and defined $pointer->[0] and !length $pointer->[0];
+  $self->_get($self->schema->data, $pointer, '');
+}
+
+sub _get {
+  my ($self, $data, $path, $pos, $cb) = @_;
   my $tied;
 
-  return $data unless ref $pointer or $pointer =~ s!^/!!;
-  $pointer = [length $pointer ? (split '/', $pointer, -1) : ($pointer)] unless ref $pointer;
-  for my $p (@$pointer) {
+  while (@$path) {
+    my $p = shift @$path;
+
+    unless (defined $p) {
+      my $i = 0;
+      return Mojo::Collection->new(map { $self->_get($_->[0], [@$path], _path($pos, $_->[1]), $cb) }
+          ref $data eq 'ARRAY' ? map { [$_, $i++] }
+          @$data : ref $data eq 'HASH' ? map { [$data->{$_}, $_] } sort keys %$data : [$data, '']);
+    }
+
     $p =~ s!~1!/!g;
     $p =~ s/~0/~/g;
+    $pos = _path($pos, $p) if $cb;
 
     if (ref $data eq 'HASH' and exists $data->{$p}) {
       $data = $data->{$p};
@@ -147,13 +160,22 @@ sub get {
     $data = $tied->schema if ref $data eq 'HASH' and $tied = tied %$data;
   }
 
+  return $cb->($data, $pos) if $cb;
   return $data;
 }
 
+sub joi {
+  return JSON::Validator::Joi->new unless @_;
+  my ($data, $joi) = @_;
+  return $joi->validate($data, $joi);
+}
+
 sub load_and_validate_schema {
   my ($self, $spec, $args) = @_;
-  $spec = $self->_reset->_resolve($spec);
-  my @errors = $self->new(%$self)->schema($args->{schema} || SPECIFICATION_URL)->validate($spec);
+  my $schema = $args->{schema} || SPECIFICATION_URL;
+  $self->version($1) if !$self->{version} and $schema =~ /draft-0+(\w+)/;
+  $spec = $self->_resolve($spec);
+  my @errors = $self->new(%$self)->schema($schema)->validate($spec);
   confess join "\n", "Invalid JSON specification $spec:", map {"- $_"} @errors if @errors;
   $self->{schema} = Mojo::JSON::Pointer->new($spec);
   $self;
@@ -162,7 +184,7 @@ sub load_and_validate_schema {
 sub schema {
   my $self = shift;
   return $self->{schema} unless @_;
-  $self->{schema} = Mojo::JSON::Pointer->new($self->_reset->_resolve(shift));
+  $self->{schema} = Mojo::JSON::Pointer->new($self->_resolve(shift));
   return $self;
 }
 
@@ -188,16 +210,19 @@ sub validate_json {
 
 sub _build_formats {
   return {
-    'date-time' => \&_is_date_time,
-    'email'     => \&_is_email,
-    'hostname'  => VALIDATE_HOSTNAME ? \&Data::Validate::Domain::is_domain : \&_is_domain,
-    'ipv4'      => VALIDATE_IP ? \&Data::Validate::IP::is_ipv4 : \&_is_ipv4,
-    'ipv6'      => VALIDATE_IP ? \&Data::Validate::IP::is_ipv6 : \&_is_ipv6,
-    'regex'     => \&_is_regex,
-    'uri'       => \&_is_uri,
+    'date-time'     => \&_is_date_time,
+    'email'         => \&_is_email,
+    'hostname'      => VALIDATE_HOSTNAME ? \&Data::Validate::Domain::is_domain : \&_is_domain,
+    'ipv4'          => VALIDATE_IP ? \&Data::Validate::IP::is_ipv4 : \&_is_ipv4,
+    'ipv6'          => VALIDATE_IP ? \&Data::Validate::IP::is_ipv6 : \&_is_ipv6,
+    'regex'         => \&_is_regex,
+    'uri'           => \&_is_uri,
+    'uri-reference' => \&_is_uri_reference,
   };
 }
 
+sub _id_key { $_[0]->version < 7 ? 'id' : '$id' }
+
 sub _load_schema {
   my ($self, $url) = @_;
 
@@ -225,14 +250,14 @@ sub _load_schema {
   if (-e $file) {
     $file = $file->realpath;
     warn "[JSON::Validator] Loading schema from file: $file\n" if DEBUG;
-    return $self->_load_schema_from_text(\$file->slurp), $url;
+    return $self->_load_schema_from_text(\$file->slurp), CASE_TOLERANT ? lc $file : $file;
   }
   elsif ($file =~ m!^/!) {
     warn "[JSON::Validator] Loading schema from URL $url\n" if DEBUG;
     return $self->_load_schema_from_url(Mojo::URL->new($url)->fragment(undef)), "$url";
   }
 
-  confess "Unable to load schema '$url'";
+  confess "Unable to load schema '$url' ($file)";
 }
 
 sub _load_schema_from_text {
@@ -251,8 +276,9 @@ sub _load_schema_from_text {
     return $v;
   };
 
-  local $YAML::Syck::ImplicitTyping = 1;
-  return $visit->($self->coerce(1)->_yaml_module->can('Load')->($$text));
+  local $YAML::Syck::ImplicitTyping = 1;            # Not in use
+  local $YAML::XS::Boolean          = 'JSON::PP';
+  return $visit->($self->_yaml_module->can('Load')->($$text));
 }
 
 sub _load_schema_from_url {
@@ -273,7 +299,10 @@ sub _load_schema_from_url {
   confess "GET $url == $err" if DEBUG and $err;
   die "[JSON::Validator] GET $url == $err" if $err;
 
-  if ($cache_path and $cache_path ne $BUNDLED_CACHE_DIR and -w $cache_path) {
+  if (  $cache_path
+    and ($cache_path ne $BUNDLED_CACHE_DIR or $ENV{JSON_VALIDATOR_CACHE_ANYWAYS})
+    and -w $cache_path)
+  {
     $cache_file = path $cache_path, $cache_file;
     warn "[JSON::Validator] Caching $url to $cache_file\n" unless $ENV{HARNESS_ACTIVE};
     $cache_file->spurt($tx->res->body);
@@ -327,20 +356,18 @@ sub _report_schema {
   push @{$self->{report}}, [(('  ') x $self->{grouped}) . ('<<<'), $path || '/', $type, D $schema];
 }
 
-sub _reset {
-  delete $_[0]->{schemas}{''};
-  $_[0]->{level} = 0;
-  $_[0];
-}
-
 # _resolve() method is used to convert all "id" into absolute URLs and
 # resolve all the $ref's that we find inside JSON Schema specification.
 sub _resolve {
   my ($self, $schema) = @_;
+  my $id_key = $self->_id_key;
   my ($id, $resolved, @refs);
 
+  local $self->{level} = $self->{level} || 0;
+  delete $_[0]->{schemas}{''} unless $self->{level};
+
   if (ref $schema eq 'HASH') {
-    $id = $schema->{id} // '';
+    $id = $schema->{$id_key} // '';
     return $resolved if $resolved = $self->{schemas}{$id};
   }
   elsif ($resolved = $self->{schemas}{$schema // ''}) {
@@ -348,14 +375,23 @@ sub _resolve {
   }
   else {
     ($schema, $id) = $self->_load_schema($schema);
-    $id = $schema->{id} if $schema->{id};
+    $id = $schema->{$id_key} if $schema->{$id_key};
   }
 
-  if (!$self->{level}++ and my $id = $schema->{id}) {
-    confess "Root schema cannot have a fragment in the 'id'. ($id)" if $id =~ /\#./;
-    confess "Root schema cannot have a relative 'id'. ($id)" if $id and $id !~ /^\w+:/;
+  unless ($self->{level}) {
+    my $rid = $schema->{$id_key} // $id;
+    if ($rid) {
+      confess "Root schema cannot have a fragment in the 'id'. ($rid)" if $rid =~ /\#./;
+      confess "Root schema cannot have a relative 'id'. ($rid)"
+        unless $rid =~ /^\w+:/
+        or -e $rid
+        or $rid =~ m!^/!;
+    }
+    warn sprintf "[JSON::Validator] Using root_schema_url of '$rid'\n" if DEBUG;
+    $self->{root_schema_url} = $rid;
   }
 
+  $self->{level}++;
   $self->_register_schema($schema, $id);
 
   my @topics = ([$schema, Mojo::URL->new($id)]);
@@ -368,8 +404,8 @@ sub _resolve {
     elsif (UNIVERSAL::isa($topic, 'HASH')) {
       push @refs, [$topic, $base] and next if $topic->{'$ref'} and !ref $topic->{'$ref'};
 
-      if ($topic->{id} and !ref $topic->{id}) {
-        my $fqn = Mojo::URL->new($topic->{id});
+      if ($topic->{$id_key} and !ref $topic->{$id_key}) {
+        my $fqn = Mojo::URL->new($topic->{$id_key});
         $fqn = $fqn->to_abs($base) unless $fqn->is_abs;
         $self->_register_schema($topic, $fqn->to_string);
       }
@@ -406,7 +442,7 @@ sub _resolve_ref {
     ($base, $pointer) = split /#/, $fqn, 2;
     $other = $self->_resolve($base);
 
-    if (length $pointer) {
+    if (defined $pointer and length $pointer) {
       $other = Mojo::JSON::Pointer->new($other)->get($pointer)
         or confess qq[Possibly a typo in schema? Could not find "$pointer" in "$base" ($ref)];
     }
@@ -415,25 +451,22 @@ sub _resolve_ref {
   tie %$topic, 'JSON::Validator::Ref', $other, $topic->{'$ref'}, $fqn;
 }
 
-# This code is from Data::Dumper::format_refaddr()
-sub _seen {
-  my $self = shift;
-  my $key = join ':', shift, map { pack 'J', refaddr $_ } @_;
-  return $self->{seen}{$key}++;
-}
-
 sub _validate {
   my ($self, $data, $path, $schema) = @_;
-  my ($type, @errors);
+  my ($seen_addr, $type, @errors);
 
   $schema = $self->_ref_to_schema($schema) if $schema->{'$ref'};
+  $seen_addr = refaddr $schema;
+  $seen_addr .= ':' . (ref $data ? refaddr $data : "s:$data") if defined $data;
 
   # Avoid recursion
-  if (ref $data and !_is_blessed_boolean($data) and $self->_seen($schema, $data)) {
+  if ($self->{seen}{$seen_addr}) {
     $self->_report_schema($path || '/', 'seen', $schema) if REPORT;
-    return;
+    return @{$self->{seen}{$seen_addr}};
   }
 
+  $self->{seen}{$seen_addr} = \@errors;
+
   # Make sure we validate plain data and not a perl object
   $data = $data->TO_JSON if blessed $data and UNIVERSAL::can($data, 'TO_JSON');
   $type = $schema->{type} || _guess_schema_type($schema, $data);
@@ -451,7 +484,7 @@ sub _validate {
   }
 
   if ($schema->{enum}) {
-    @errors = $self->_validate_type_enum($data, $path, $schema);
+    push @errors, $self->_validate_type_enum($data, $path, $schema);
     $self->_report_errors($path, 'enum', \@errors) if REPORT;
     return @errors if @errors;
   }
@@ -565,15 +598,12 @@ sub _validate_type_enum {
   my $m    = S $data;
 
   for my $i (@$enum) {
-    return
-      if !(defined $data and $self->_validate_type_boolean($data, $path))
-      and _is_true($data) == _is_true($i);
     return if $m eq S $i;
   }
 
   local $" = ', ';
   return E $path, sprintf 'Not in enum list: %s.', join ', ',
-    map { ref $_ ? Mojo::JSON::encode_json($_) : $_ } @$enum;
+    map { (!defined or ref) ? Mojo::JSON::encode_json($_) : $_ } @$enum;
 }
 
 sub _validate_type_const {
@@ -743,16 +773,9 @@ sub _validate_type_object {
     $additional = {} unless UNIVERSAL::isa($additional, 'HASH');
     $rules{$_} ||= [$additional] for @dkeys;
   }
-  else {
-    # Special case used internally when validating schemas: This module adds "id"
-    # on the top level which might conflict with very strict schemas, so we have to
-    # remove it again unless there's a rule.
-    local $rules{id} = 1 if !$path and exists $data->{id};
-
-    if (my @k = grep { !$rules{$_} } @dkeys) {
-      local $" = ', ';
-      return E $path, "Properties not allowed: @k.";
-    }
+  elsif (my @k = grep { !$rules{$_} } @dkeys) {
+    local $" = ', ';
+    return E $path, "Properties not allowed: @k.";
   }
 
   for my $k (sort keys %required) {
@@ -763,18 +786,13 @@ sub _validate_type_object {
 
   for my $k (sort keys %rules) {
     for my $r (@{$rules{$k}}) {
-      if (!exists $data->{$k} and (UNIVERSAL::isa($r, 'HASH') and exists $r->{default})) {
-
-        #$data->{$k} = $r->{default}; # TODO: This seems to fail when using oneOf and friends
-      }
-      elsif (exists $data->{$k}) {
-        my @e = $self->_validate($data->{$k}, _path($path, $k), $r);
-        push @errors, @e;
-        push @errors, $self->_validate_type_enum($data->{$k}, _path($path, $k), $r)
-          if $r->{enum} and !@e;
-        push @errors, $self->_validate_type_const($data->{$k}, _path($path, $k), $r)
-          if $r->{const} and !@e;
-      }
+      next unless exists $data->{$k};
+      my @e = $self->_validate($data->{$k}, _path($path, $k), $r);
+      push @errors, @e;
+      push @errors, $self->_validate_type_enum($data->{$k}, _path($path, $k), $r)
+        if $r->{enum} and !@e;
+      push @errors, $self->_validate_type_const($data->{$k}, _path($path, $k), $r)
+        if $r->{const} and !@e;
     }
   }
 
@@ -829,13 +847,6 @@ sub _cmp {
   return "";
 }
 
-sub _definitions_name {
-  local $_ = "$_[1]";
-  s!\#!-!g;
-  s![^\w-]!_!g;
-  return "_$_";
-}
-
 sub _expected {
   my $type = _guess_data_type($_[1]);
   return "Expected $_[0] - got different $type." if $_[0] =~ /\b$type\b/;
@@ -924,13 +935,6 @@ sub _is_ipv4 {
 
 sub _is_ipv6 { warn "Data::Validate::IP is not installed"; return; }
 
-sub _is_true {
-  local $_ = $_[0];
-  return 0 + $_ if ref $_ and !blessed $_;
-  return 0 if !$_ or /^(n|false|off)/i;
-  return 1;
-}
-
 sub _is_blessed_boolean {
   return 0 if !blessed $_[0];
   return 1 if UNIVERSAL::isa($_[0], 'JSON::PP::Boolean') or "$_[0]" eq "1" or !$_[0];
@@ -966,6 +970,17 @@ sub _is_uri {
   return 1;
 }
 
+sub _is_uri_reference {
+  return unless defined $_[0];
+  return unless $_[0] =~ m!^(([^:/?#]+):)?(//([^/?#]*))?([^?#]*)(\?([^#]*))?(#(.*))?!;
+
+  my ($scheme, $auth_host, $path, $query, $fragment) = map { $_ // '' } ($2, $4, $5, $7, $9);
+  return _invalid('Path cannot not start with //.') if $path =~ m!^//!;
+  return 1 if length $path;
+  return _is_uri($_[0]);
+  return 1;
+}
+
 sub _merge_errors {
   join ' ', map {
     my $e = $_;
@@ -988,15 +1003,8 @@ sub _uniq {
 # Please report if you need to manually monkey patch this function
 # https://github.com/jhthorsen/json-validator/issues
 sub _yaml_module {
-  state $yaml_module = do {
-    require List::Util;
-    my @modules = qw(YAML::XS YAML::Syck);                                  # subject to change
-    my $module = (List::Util::first { eval "require $_;1" } @modules)[0];
-    die "Need to install one of these YAML modules: @modules (YAML::XS is recommended)"
-      unless $module;
-    warn "[JSON::Validator] Using $module to parse YAML\n" if DEBUG;
-    $module;
-  };
+  state $yaml_module = eval qq[use YAML::XS 0.67; "YAML::XS"]
+    || die "[JSON::Validator] The optional YAML::XS module is missing or could not be loaded: $@";
 }
 
 1;
@@ -1009,7 +1017,7 @@ JSON::Validator - Validate data against a JSON schema
 
 =head1 VERSION
 
-1.08
+2.08
 
 =head1 SYNOPSIS
 
@@ -1050,12 +1058,14 @@ which will replace L<Mojolicious::Plugin::Swagger2>.
 =head2 Supported schema formats
 
 L<JSON::Validator> can load JSON schemas in multiple formats: Plain perl data
-structured (as shown in L</SYNOPSIS>) or files on disk/web in the JSON/YAML
-format. The JSON parsing is done using L<Mojo::JSON>, while the YAML parsing
-is done with an optional modules which need to be installed manually.
-L<JSON::Validator> will look for the YAML modules in this order: L<YAML::XS>,
-L<YAML::Syck>. The order is set by which module that performs the best, so it
-might change in the future.
+structured (as shown in L</SYNOPSIS>), JSON or YAML. The JSON parsing is done
+with L<Mojo::JSON>, while YAML files require the optional module L<YAML::XS> to
+be installed.
+
+IMPORTANT! L<YAML::Syck> is not supported in L<JSON::Validator> 2.00. Only
+L<YAML::XS> is supported, since it has proper boolean handling. Look for
+C<$YAML::XS::Boolean> in the documentation to see what is recognized as
+booleans.
 
 =head2 Resources
 
@@ -1092,11 +1102,13 @@ Here is the list of the bundled specifications:
 
 =over 2
 
-=item * JSON schema, draft 4
+=item * JSON schema, draft 4, 6, 7
 
 Web page: L<http://json-schema.org>
 
-C<$ref>: L<http://json-schema.org/draft-04/schema#>
+C<$ref>: L<http://json-schema.org/draft-04/schema#>,
+L<http://json-schema.org/draft-06/schema#>,
+L<http://json-schema.org/draft-07/schema#>.
 
 =item * JSON schema for JSONPatch files
 
@@ -1180,6 +1192,17 @@ welcome!
 
 =head1 FUNCTIONS
 
+=head2 joi
+
+  use JSON::Validator "joi";
+  my $joi = joi;
+  my @errors = joi($data, $joi); # same as $joi->validate($data);
+
+Used to construct a new L<JSON::Validator::Joi> object or perform validation.
+
+Note that this function iS EXPERIMENTAL. See L<JSON::Validator::Joi> for more
+details.
+
 =head2 validate_json
 
   use JSON::Validator "validate_json";
@@ -1262,6 +1285,16 @@ Note that the default L<Mojo::UserAgent> will detect proxy settings and have
 L<Mojo::UserAgent/max_redirects> set to 3. (These settings are EXPERIMENTAL
 and might change without a warning)
 
+=head2 version
+
+  $int = $self->version;
+  $self = $self->version(7);
+
+Used to set the JSON Schema version to use. Will be set automatically when
+using L</load_and_validate_schema>, unless already set.
+
+Note that this attribute is EXPERIMENTAL and might change without a warning.
+
 =head1 METHODS
 
 =head2 bundle
@@ -1295,19 +1328,12 @@ Default is to use the value from the L</schema> attribute.
 
   $self = $self->coerce(booleans => 1, numbers => 1, strings => 1);
   $self = $self->coerce({booleans => 1, numbers => 1, strings => 1});
-  $self = $self->coerce(1) # enable all
   $hash = $self->coerce;
 
 Set the given type to coerce. Before enabling coercion this module is very
 strict when it comes to validating types. Example: The string C<"1"> is not
 the same as the number C<1>, unless you have coercion enabled.
 
-WARNING! Enabling coercion might hide bugs in your api, which would have been
-detected if you were strict. For example JavaScript is very picky on a number
-being an actual number. This module tries it best to convert the data on the
-fly into the proper value, but this means that you unit tests might be ok,
-but the client side libraries (that care about types) might break.
-
 Loading a YAML document will enable "booleans" automatically. This feature is
 experimental, but was added since YAML has no real concept of booleans, such
 as L<Mojo::JSON> or other JSON parsers.
@@ -1319,6 +1345,7 @@ for more details.
 =head2 get
 
   $sub_schema = $self->get("/x/y");
+  $sub_schema = $self->get(["x", "y"]);
 
 Extract value from L</schema> identified by the given JSON Pointer. Will at the
 same time resolve C<$ref> if found. Example:
@@ -1330,6 +1357,9 @@ same time resolve C<$ref> if found. Example:
 
 This method is EXPERIMENTAL.
 
+The argument can also be an array-ref with the different parts of the pointer
+as each elements.
+
 =head2 load_and_validate_schema
 
   $self = $self->load_and_validate_schema($schema, \%args);
